<template>
  <div class="bbsl-common-page">
    <div class="bbsl-table-search">
      <el-form v-model="queryParam" label-width="auto">
        <el-row :gutter="10">
          <el-col :span="5">
            <el-form-item label="消息状态">
              <el-select placeholder="请选择消息状态" v-model="queryParams.receiveTime">
                <el-option v-for="item in receiveTimeOptions" :key="item.value" :label="item.name"
                  :value="item.value" />
              </el-select>
            </el-form-item>
          </el-col>
          <el-col :span="4">
            <el-button type="primary" icon="el-icon-search" @click="handleSearch">搜索
            </el-button>
            <el-button icon="el-icon-refresh" @click="handleReset">重置
            </el-button>
          </el-col>
        </el-row>
      </el-form>
    </div>
    <div class="bbsl-table-content" ref="contentPanel">
      <div class="bbsl-btn-content" ref="btnPanel">
        <el-button icon="el-icon-s-claim" type="warning" plain class="btn" :disabled="!multipleSelection.length"
          @click="markAsRead()">
          标为已读
        </el-button>
        <el-button icon="el-icon-delete" type="danger" plain class="btn btn-delete"
          :disabled="!multipleSelection.length" @click="handleDelete()">
          删除
        </el-button>
      </div>
      <el-table border size="mini" v-loading="loading" ref="multipleSelection" :data="tableData"
        :style="{ width: '100%' }" :max-height="tableMaxHeight" :cell-style="{ 'text-align': 'center' }"
        :header-cell-style="{ 'text-align': 'center' }" @selection-change="handleSelectionChange"
        @sort-change="sortChange" row-key="id">
        <el-table-column reserve-selection type="selection" width="55" align="center"></el-table-column>
        <el-table-column type="index" width="70" label="序号" align="center">
        </el-table-column>
        <el-table-column prop="subject" show-overflow-tooltip min-width="220" align="center" label="标题" />
        <el-table-column prop="receiveTime" show-overflow-tooltip min-width="90" align="center" label="消息状态">
          <template slot-scope="{row:{receiveTime}}">
            <el-tag v-if="receiveTime" type="warning" size="medium">已读</el-tag>
            <el-tag v-else type="danger">未读</el-tag>
          </template>
        </el-table-column>
        <el-table-column prop="owner" show-overflow-tooltip min-width="90" align="center" label="发件人">
        </el-table-column>
        <el-table-column v-if="tableData.some(i => i.receiveTime)" prop="receiveTime" show-overflow-tooltip
          align="center" label="已读时间" min-width="120" />
        <el-table-column prop="createTime" show-overflow-tooltip align="center" label="发信时间"
          :min-width="tableData.some(i => i.receiveTime) ? '120' : '180'" />
        <el-table-column show-overflow-tooltip min-width="120" align="center" fixed="right" label="操作">
          <template v-slot="{ row }">
            <el-link v-if="!row.receiveTime" type="warning" :underline="false" style="margin-right: 15px;"
              @click="markAsRead(row)">
              标为已读
            </el-link>
            <el-link type="primary" :underline="false" style="margin-right: 15px;"
              @click="showDialog('isShowDetail', row)">
              详情
            </el-link>
            <el-link type="danger" :underline="false" style="margin-right: 15px;" @click="handleDelete(row)">
              删除
            </el-link>
          </template>
        </el-table-column>
      </el-table>
      <el-pagination align="right" background class="bbsl-pagination" ref="paginationPanel"
        @size-change="handleSizeChange" @current-change="handleCurrentChange" :current-page="queryParams.page"
        :page-sizes="[20, 30, 40, 50]" :page-size="queryParams.pageSize"
        layout="total, sizes, prev, pager, next, jumper" :total="queryParams.total"></el-pagination>
    </div>

    <!--  详情弹层  -->
    <detail-dialog :isShowDetail.sync="isShowDetail" width="40%" title="消息通知" :rowInfo="rowInfo"
      @close-dialog="closeDetail" />

  </div>
</template>

<script>
import { TableMixin } from '@/mixins/tableMixin'
import { messageReceiver, removeMessage, markMessage } from '@/api/MessageReminder'
import DetailDialog from './components/DetailDialog.vue'

export default {
  name: 'SecurityReminder',
  components: { DetailDialog },
  mixins: [TableMixin],
  data() {
    return {
      isShowDetail: false,
      tableMaxHeight: '600px',
      loading: false,
      tableLoading: false,
      tableData: [],
      queryParams: {
        page: 1,
        pageSize: 20,
        total: 0,
        subject: '安全',
        receiveTime: '1'
      },
      sorter: [
        {
          direction: 'ASC',
          property: 'createTime'
        },
        {
          direction: '',
          property: 'subject'
        },
        {
          direction: '',
          property: 'owner'
        }
      ],
      receiveTimeOptions: [
        { name: '未读', value: '1' },
        { name: '已读', value: '2' }
      ],
      rowInfo: {}
    }
  },
  computed: {
    // 加工为后端需要的请求格式
    computedQuery() {
      /*
       * 定义 escapeOpt 字段对应字典名 对象
       *  为重新加工结构赋值的新请求配置参数
       * */
      const querys = [
        { operation: 'EQUAL', property: 'receiveTime', relation: 'AND', group: 'filterTime' },
        { operation: 'LIKE', property: 'subject', relation: 'OR', group: 'quick' }
      ]
      const { pageSize, page, ...item } = this.queryParams
      return {
        pageBean: { pageSize, page },
        querys: querys.flatMap(i => {
          const value = item[i.property]
          if (Array.isArray(value) && !value.length) return []
          if (value === '') return []
          return { group: 'main', ...i, value }
        }),
        sorter: this.sorter.filter(i => i.direction !== '')
      }
    }
  },
  created() {
  },
  methods: {

    // 获取list
    async loadData() {
      this.tableLoading = true
      const { data: { rows, total } } = await messageReceiver(this.computedQuery)
      this.tableData = rows
      this.queryParams.total = total
      this.tableLoading = false
    },

    // 删除
    handleDelete(row) {
      this.$confirm('确定删除选中的消息吗？', '提示', {
        confirmButtonText: '确认',
        cancelButtonText: '取消',
        type: 'warning'
      }).then(async () => {
        let ids = (row ? row.id : this.multipleSelection.join()) || ''
        if (!ids) return
        try {
          const { data: { state, message } } = await removeMessage(ids)
          if (state) {
            this.queryParams.page = 1
            await this.loadData()
            this.$message.success(message)
          } else {
            this.$message.error(message)
          }
        } catch {
          this.$message.error('删除消息失败')
        }
      }).catch(_ => _)
    },

    // 标记为已读
    markAsRead(row) {
      this.$confirm('确定标记选中的消息为已读状态吗？', '提示', {
        confirmButtonText: '确认',
        cancelButtonText: '取消',
        type: 'warning'
      }).then(async () => {
        let ids = (row ? row.id : this.multipleSelection.join()) || ''
        if (!ids) return
        try {
          const { data: { state, message } } = await markMessage(ids)
          if (state) {
            await this.loadData()
            this.$message.success(message)
          } else {
            this.$message.error(message)
          }
        } catch {
          this.$message.error('标记失败')
        } finally {
          this.$refs.multipleSelection && this.$refs.multipleSelection.clearSelection()
        }
      }).catch(_ => _)
    },

    // 顶部搜索
    handleSearch() {
      this.loadData()
    },

    // 重置
    handleReset() {
      this.queryParams = this.$options.data().queryParams
      this.loadData()
    },

    //分页页数
    handleCurrentChange(page) {
      this.queryParams.page = page
      this.loadData()
    },

    //分页大小
    handleSizeChange(pageSize) {
      this.queryParams.pageSize = pageSize
      this.loadData()
    },

    // 排序切换
    sortChange({ prop: property, order }) {
      let direction = ''
      switch (order) {
        case 'ascending':
          direction = 'ASC'
          break
        case 'descending':
          direction = 'DESC'
          break
      }
      this.sorter.forEach(i => {
        if (i.property !== property) return i.direction = ''
        i.direction = direction
      })
      this.loadData()
    },

    // 展示弹层
    showDialog(dialog, rowInfo = {}) {
      this[dialog] = true
      this.rowInfo = rowInfo // 保存点击行信息
    },

    //  关闭详情弹层
    closeDetail() {
      if (!this.rowInfo.receiveTime) {
        this.queryParams.page = 1
        this.loadData()
      }
    }

  }
}
</script>
<style lang="scss" scoped>
.type-tag {
  margin: 0 2px;
}
</style>
